(require 'arc-mode)

(setq archive-7z-program "c:/Program Files/7-Zip/7z.exe")
(defvar archive-utf-python-script
  (format "\
import sys, base64, subprocess

PROG_7Z = '%s'
MODE    = sys.argv[1]
for i in range(2, len(sys.argv)):
    idx = sys.argv[i].rfind('/')
    sys.argv[i] = sys.argv[i][idx+1:]
    sys.argv[i] = base64.b64decode(sys.argv[i]).decode()
ARCHIVE = sys.argv[2]
SUBFILES = sys.argv[3:]
SUBFILE = SUBFILES[0] if SUBFILES else ''

if MODE == 'x':
    subprocess.run([PROG_7Z, 'x', '-so', ARCHIVE, SUBFILE])
elif MODE == 'u':
    subprocess.run([PROG_7Z, 'u', ARCHIVE, SUBFILE])
elif MODE == 'd':
    subprocess.run([PROG_7Z, 'd', ARCHIVE] + SUBFILES)
elif MODE == 'l':
    subprocess.run([PROG_7Z, 'l', ARCHIVE])
" archive-7z-program))

(defvar archive-utf-extract `("python" "-c" ,archive-utf-python-script "x"))
(defvar archive-utf-expunge `("python" "-c" ,archive-utf-python-script "d"))
(defvar archive-utf-update  `("python" "-c" ,archive-utf-python-script "u"))

(defun archive-utf-encode (str)
  (base64-encode-string (encode-coding-string str 'utf-8)))

(defun archive-zip-extract (archive name)
  (let ((default-directory temporary-file-directory))
    (archive-extract-by-stdout
     (archive-utf-encode (expand-file-name archive))
     (archive-utf-encode name)
     archive-utf-extract)))

(defun archive-zip-expunge (archive files)
  (let ((default-directory temporary-file-directory))
    (archive-*-expunge
     (archive-utf-encode (expand-file-name archive))
     (mapcar #'archive-utf-encode files)
     archive-utf-expunge)))

(defun archive-zip-write-file-member (archive descr)
  (let ((default-directory temporary-file-directory)
	(archive-file-name-coding-system 'archive-base64))
    (archive-*-write-file-member
     (archive-utf-encode (expand-file-name archive))
     descr
     archive-utf-update)))

(define-coding-system 'archive-base64
  "base64 encoding"
  :mnemonic ?B
  :coding-type 'undecided
  :post-read-conversion 'archive-base64-post-read-conversion
  :pre-write-conversion 'archive-base64-pre-write-conversion)

(defun archive-base64-post-read-conversion (len)
  (let ((pos (point))
	(buffer-modified-p (buffer-modified-p))
	last-coding-system-used)
    (prog1
	(save-restriction
	  (narrow-to-region pos (+ pos len))
	  (base64-decode-region pos (point-max))
	  (decode-coding-region pos (point-max)'utf-8))
      (set-buffer-modified-p buffer-modified-p))))

(defun archive-base64-pre-write-conversion (from to)
  (let ((buf (current-buffer)))
    (set-buffer (generate-new-buffer " *temp*"))
    (if (stringp from)
	(insert from)
      (insert-buffer-substring buf from to))
    (let (last-coding-system-used)
      (encode-coding-region 1 (point-max) 'utf-8)
      (base64-encode-region 1 (point-max)))
    nil))
